package main

import (
	"encoding/json"
	"fmt"
	"io/fs"
	"log"
	"os"
	"os/exec"
	"path"
	"path/filepath"
	"uploader/download"

	"gioui.org/app"
	"gioui.org/io/system"
	"gioui.org/layout"
	"gioui.org/op"
	"gioui.org/text"
	"gioui.org/unit"
	"gioui.org/widget"
	"gioui.org/widget/material"
	"github.com/jessevdk/go-flags"
	"infish.cn/comm"
)

// 启动程序
// 下载文件->成功->失败
// 下载成功->通知nats关闭原程序
// 安装->成功->失败
// 成功启动程序
// 失败还原程序，关闭更新应用

// Define the progress variables, a channel and a variable

var progress float32
var progressIncrementer chan float32

var GAppOption = &AppOption{}

type AppOption struct {
	NatsPort   int    `short:"np" long:"np" description:"nats port"`
	AppExePath string `short:"ap" long:"ap" description:"app exe path"`
	Url        string `short:"url" long:"url" description:"url"`
}

func (o *AppOption) Parse() {
	flags.NewParser(o, flags.Default|flags.IgnoreUnknown).Parse()
}

func main() {
	GAppOption.Parse()
	err := download.DownloadExe(GAppOption.Url, GAppOption.AppExePath)
	if err != nil {
		fmt.Println(err)
		return
	}
	remoteCnn, _ := comm.NewNatsBus(fmt.Sprintf("nats://localhost:%d", GAppOption.NatsPort), 1, 1, []*comm.NatsStreamWather{})
	if remoteCnn != nil {
		d, _ := json.Marshal(map[string]string{"shortcut": ""})
		remoteCnn.Publish("lancher.resume", d)
		remoteCnn.GetNatsConn().Flush()
	}
	appExeDir, _ := filepath.Split(GAppOption.AppExePath)
	os.Rename(GAppOption.AppExePath, fmt.Sprintf("%s/lancherlast", appExeDir))

	fmt.Println("need restart ", GAppOption.AppExePath)
	// 重启
	cmd := exec.Command(GAppOption.AppExePath)
	cmd.Stdout = os.Stdout
	cmd.Stderr = os.Stderr
	err = cmd.Start()
	if err != nil {
		fmt.Printf("failed to call cmd.Start(): %v", err)
		return
	}
	os.WriteFile(path.Join(appExeDir, "pid"), []byte(fmt.Sprintf("%d", cmd.Process.Pid)), fs.FileMode(os.O_TRUNC))
	log.Printf("pid: %d", cmd.Process.Pid)
	lancherbak := fmt.Sprintf("%s/%s", appExeDir, "lancherlast")
	err = os.Remove(lancherbak)
	if err != nil {
		fmt.Println(err)
	}
	cmd.Process.Wait()

	// // Setup a separate channel to provide ticks to increment progress
	// progressIncrementer = make(chan float32)
	// go func() {
	// 	for {
	// 		time.Sleep(time.Second / 25)
	// 		progressIncrementer <- 0.004
	// 	}
	// }()
	// // task := make(map[string]string)
	// // task["https://dl.softmgr.qq.com/original/game/WeGameSetup3.32.4.6183_gjwegame_0_0.exe"] = "wegame.exe"
	// // for k, v := range task {
	// // 	// wg.Add(1)
	// // 	download.DownloadFile(k, v)
	// // }

	// go func() {
	// 	// create new window
	// 	w := app.NewWindow(
	// 		app.Title("uploader"),
	// 		app.Size(unit.Dp(600), unit.Dp(400)),
	// 	)
	// 	if err := draw(w); err != nil {
	// 		log.Fatal(err)
	// 	}
	// 	os.Exit(0)
	// }()

	// app.Main()
}

type C = layout.Context
type D = layout.Dimensions

func draw(w *app.Window) error {
	// ops are the operations from the UI
	var ops op.Ops

	// startButton is a clickable widget
	var startButton widget.Clickable

	// is the egg boiling?
	var boiling bool

	// th defines the material design style
	th := material.NewTheme([]text.FontFace{})

	for {
		select {
		// listen for events in the window.
		case e := <-w.Events():

			// detect what type of event
			switch e := e.(type) {

			// this is sent when the application should re-render.
			case system.FrameEvent:
				gtx := layout.NewContext(&ops, e)
				// Let's try out the flexbox layout concept
				if startButton.Clicked() {
					boiling = !boiling
				}

				layout.Flex{
					// Vertical alignment, from top to bottom
					Axis: layout.Vertical,
					// Empty space is left at the start, i.e. at the top
					Spacing: layout.SpaceStart,
				}.Layout(gtx,
					layout.Rigid(
						func(gtx C) D {
							bar := material.ProgressBar(th, progress)
							return bar.Layout(gtx)
						},
					),
					layout.Rigid(
						func(gtx C) D {
							// We start by defining a set of margins
							margins := layout.Inset{
								Top:    unit.Dp(25),
								Bottom: unit.Dp(25),
								Right:  unit.Dp(25),
								Left:   unit.Dp(25),
							}
							// Then we lay out within those margins ...
							return margins.Layout(gtx,
								// ...the same function we earlier used to create a button
								func(gtx C) D {
									var text string
									if !boiling {
										text = "Start"
									} else {
										text = "Stop"
									}
									btn := material.Button(th, &startButton, text)
									return btn.Layout(gtx)
								},
							)
						},
					),
				)
				e.Frame(gtx.Ops)

			// this is sent when the application is closed.
			case system.DestroyEvent:
				return e.Err
			}

		// listen for events in the incrementor channel
		case p := <-progressIncrementer:
			if boiling && progress < 1 {
				progress += p
				w.Invalidate()
			}
		}
	}
}
